private {
    import doodle.core.backtrace;
    import doodle.core.logging;
    import doodle.core.backtrace;

    import doodle.dia.standard_tools;
    import doodle.dia.page_layer;
    import doodle.dia.grid_layer;
    import doodle.fig.diagram_layer;
    import doodle.dia.tool_layer;

    import doodle.fig.tools;

    import doodle.gtk.canvas;
    import doodle.gtk.palette;

    import gtk.Main;
    import gtk.MainWindow;
    import gtk.VBox;

    import std.stdio;
}

final class TopLevel : private IToolStackObserver {
    this(string[] args) {
        Main.init(args);
        auto window = new MainWindow("Doodle");
        auto vbox = new VBox(false, 0);

        auto palette = new Palette;
        _palette = palette;
        IPalette.Item[] items = [
        { "select.svg",    "Select",    "Select and modify elements", &paletteCallback },
        { "rectangle.svg", "Rectangle", "Create rectangle",           &paletteCallback },
        { "ellipse.svg",   "Ellipse",   "Create ellipse",             &paletteCallback },
        { "polyline.svg",  "Polyline",  "Create polyline",            &paletteCallback }
        ];
        _palette.configure(items);
        vbox.packStart(palette, false, false, 0);

        Tool[] tools;
        tools ~= new PanTool;
        tools ~= new ZoomTool;
        tools ~= new SelectTool;
        auto toolLayer = new ToolLayer(tools, this);
        _toolStack = toolLayer;

        auto diagramLayer = new DiagramLayer;
        _diagram = diagramLayer;

        auto gridLayer = new GridLayer;

        Layer[] layers;
        layers ~= new PageLayer;
        layers ~= gridLayer;
        layers ~= toolLayer;
        auto canvas = new Canvas(layers, toolLayer, gridLayer, 120.0);

        vbox.packStart(canvas, true, true, 0);
        window.add(vbox);
        window.setDefaultSize(640, 580);
        window.showAll();
        Main.run();
    }

    void toolChanged(int index) {      // FIXME dmd bug? have to make this public
        message("Tool changed %s", index);
        _palette.activate(index);
    }

    private {
        IToolStack _toolStack;
        IPalette _palette;
        IDiagram _diagram;

        void paletteCallback(int index) {
            switch (index) {
            case 0:
                // Select
                break;
            case 1:
                // Rectangle
                _toolStack.use(new CreateRectangleTool(_diagram));
                break;
            case 2:
                // Ellipse
                break;
            case 3:
                // Polyline
                break;
            default:
                assert(0);
            }
        }

    }
}

void main(string[] args) {
    new TopLevel(args);
}
